#!/devtool/local/bin/perl
#
#    ========== licence begin LGPL
#    Copyright (C) 2002 SAP AG
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#    ========== licence end
#


use Getopt::Std;

my %RunningCrons = GetRunningCrons();

if ( !getopts('k:') ) {
    print "USAGE: nachtkill.pl [-k <release>]\n";
    print "\n";
    print "       kills running nachtRelease, without parameter shows all running jobs\n";
    print "\n";
	foreach my $Rel (keys %RunningCrons) {
		print "       nachtRelease $Rel still running (Pid:$RunningCrons{$Rel})\n";
	}
    exit 1;
}


if ( $opt_k ) {

	if ( $RunningCrons{$opt_k} ) {
		print "Killing releas $opt_k (Pid:$RunningCrons{$opt_k}))\n";
		PauseCron ($RunningCrons{$opt_k});
		KillCron ($RunningCrons{$opt_k});

		my @DBNames = GetSutDBNames($opt_k);
		foreach my $DB (@DBNames) {
			print "stop $DB\n";
	        system ("dbmcli -d $DB -u superdba,colduser db_stop");
		}

		my @RelDirs = GetRelDirs($opt_k);
		foreach my $RelDir (@RelDirs) {
			print "rm currentLogin in $RelDir\n";
	        unlink ("$RelDir/currentLogin");
		}

	} else {
		print "unknown or not running release: $opt_k\n"
	}

} else {

	if (%RunningCrons) {
		foreach my $Rel (keys %RunningCrons) {
			print "nachtRelease $Rel still running (Pid:$RunningCrons{$Rel})\n";
		}
	} else {
		print "no nachtRelease running\n";
	}

}
########################################
sub GetRunningCrons ()
{
	my %CronHash;

	my ($CronPid) = split ' ', `ps -e | grep cron`;

	my @NachtProcs = grep { s/^\s*remuser\s+(\d+)\s+$CronPid\s+.*nachtRelease\.bash\s+(\d*)\s+(nacht|weekend)\s+(32Bit|64Bit|32__64).*$/$1 $2 $4/ } `ps -ef`;

	foreach my $NachtProc (@NachtProcs) {

		my ($Pid, $Release, $BitKind) = split ( ' ', $NachtProc );
		$CronHash{$Release."_".$BitKind} = $Pid;

	}

	return %CronHash;
}

########################################
sub PauseCron ()
{
	my (@ProcList) = @_;

    while (@ProcList) {
		my $ActPid = shift @ProcList;
		kill 'STOP', $ActPid;
		push @ProcList , GetChildPids($ActPid);
	}

}

########################################
sub KillCron ()
{
	my (@ProcList) = @_;

    while (@ProcList) {
		my $ActPid = shift @ProcList;
		push @ProcList , GetChildPids($ActPid);
		kill 'KILL', $ActPid;
	}

}

########################################
sub GetChildPids ()
{
	my ($ParentPid) = @_;

	my @ChildPids;
    my @psout = `ps -ef | grep remuser | grep $ParentPid`;
    foreach my $ps ( @psout ) {
   		push (@ChildPids, $1) if ( $ps =~ /.*\s+(\d+)\s+$ParentPid\s+.+$/ );
	}

    return @ChildPids;

}

########################################
sub GetSutDBNames ()
{
	my ($RelName) = @_;

	my ($Rel,$BitKind) = $RelName =~ /(\d+)_(.*)/;

    my ($RelLength) = length $Rel;
    my @SutDBNames;
  	if (($ENV{BIT32Supported} eq "NO") or ($ENV{BIT64Supported} eq "NO")) {
	    push @SutDBNames, "SUT${Rel}";
    } elsif ( $RelLength <= 2 ) {
        if ( ($BitKind eq "32Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SUT${Rel}_32";
        }
        if ( ($BitKind eq "64Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SUT${Rel}_64";
        }
    } elsif ( $RelLength == 3 ){
        if ( ($BitKind eq "32Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SUT${Rel}32";
        }
        if ( ($BitKind eq "64Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SUT${Rel}64";
        }
	} else {
        if ( ($BitKind eq "32Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SU${Rel}32";
        }
        if ( ($BitKind eq "64Bit") or ($BitKind eq "32__64") ) {
            push @SutDBNames, "SU${Rel}64";
        }
    }

    return @SutDBNames;

}

########################################
sub GetRelDirs ()
{
	my ($RelName) = @_;

	my ($Rel,$BitKind) = $RelName =~ /(\d+)_(.*)/;

    my @RelDirs;
    if ( ($BitKind eq "32Bit") or ($BitKind eq "32__64") ) {
    	if (($ENV{BIT32Supported} eq "NO") or ($ENV{BIT64Supported} eq "NO")) {
			push @RelDirs, "/rel/${Rel}";
		} else {
			push @RelDirs, "/rel/${Rel}_32";
		}
	}

    if ( ($BitKind eq "64Bit") or ($BitKind eq "32__64") ) {
    	if (($ENV{BIT32Supported} eq "NO") or ($ENV{BIT64Supported} eq "NO")) {
			push @RelDirs, "/rel/${Rel}";
		} else {
			push @RelDirs, "/rel/${Rel}_64";
		}
    }

    return @RelDirs;

}

